/* GADMIN-DHCPD - An easy to use GTK+ frontend for ISC DHCPD.
 * Copyright (C) 2004 - 2011 Magnus Loef <magnus-swe@telia.com> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
*/



#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "widgets.h"
#include "gettext.h"
#include "allocate.h"
#include "commented.h"

extern char LEASE_FILE_BUF[1024];

#define MAX_LEASE_LINE 1024

/* Only the last occurance of a host declaration
   in dhcpd.leases should be listed */


void add_to_treeview(struct w *widgets, int col, char *line)
{
    int i = 0;
    gchar *utf8;
    char *buf = allocate(strlen(line)+100);

    /* Snip some unwanted stuff */
    for(i=0; i<strlen(line); i++)
       if( line[i]=='\n' || line[i]==';' )
         line[i]='\0';

    /* Binding state */
    if( col == 0 )
    {
	if( strstr(line, "free") )
	{
	    strcpy(buf, _("Free"));
	}
	if( strstr(line, "active") )
	{
	    strcpy(buf, _("Active"));
	}
    }
    else
    /* IP address */
    if( col == 1 )
    {
	sscanf(line, "%*s %s", buf);
    }
    else
    /* Hardware type */
    if( col == 2 )
    {
	sscanf(line, "%*s %s", buf);
    }
    else
    /* Hardware address */
    if( col == 3 )
    {
	sscanf(line, "%*s %*s %s", buf);
    }
    else
    /* Client-hostname */
    if( col == 4 )
    {
	sscanf(line, "%*s %s", buf);
	/* Snip the first and last chars: "Hostname" */
	buf[0]=' ';
	buf[strlen(buf)-1]='\0';
    }
    else
    /* Starts */
    if( col == 5 )
    {
	sscanf(line, "%*s %*s %s", buf);
    }
    else
    /* Ends */
    if( col == 6 )
    {
	sscanf(line, "%*s %*s %s", buf);
    }
    else
    /* Next binding state */
    if( col == 7 )
    {
	if( strstr(line, "free") )
	{
	    strcpy(buf, _("Free"));
	}
	if( strstr(line, "active") )
	{
	    strcpy(buf, _("Active"));
	}
    }
    else
    /* CLTT */
    if( col == 8 )
    {
	sscanf(line, "%*s %*s %s", buf);
    }
    else
    /* TSTP */
    if( col == 9 )
    {
	sscanf(line, "%*s %*s %s", buf);
    }
    else
    /* TSFP */
    if( col == 10 )
    {
	sscanf(line, "%*s %*s %s", buf);
    }
    else
    /* UUID */
    if( col == 11 )
    {
	sscanf(line, "%*s %s", buf);
    }
    else
	printf("Unhandled option: %s\n", buf);


    utf8 = g_locale_to_utf8(buf, strlen(buf), NULL, NULL, NULL);
    gtk_list_store_set(GTK_LIST_STORE(widgets->lease_store), &widgets->lease_iter, col, utf8, -1);
    if( utf8!=NULL )
      g_free(utf8);


    if( buf!=NULL )
      free(buf);
}


void populate_leases(struct w *widgets)
{
    FILE *fp;
    long file_size = 0;
    long file_pos  = 0;
    long orig_file_pos = 0;
    char *line, *buf;
    int pop_lease = 0;

    gtk_list_store_clear(widgets->lease_store);

    if((fp=fopen(LEASE_FILE_BUF, "r"))==NULL)
    {
        return;
    }
    fseek(fp, 0, SEEK_END);
    file_size = ftell(fp);
    rewind(fp);

    line = allocate(file_size+1);
    buf  = allocate(MAX_LEASE_LINE+1);

    if( file_size > 1 )
    while(fgets(line, file_size, fp)!=NULL)
    {
	if( commented(line) || strlen(line) > MAX_LEASE_LINE )
	  continue;

	/* Found a lease, save pos and see if this
	   is the last lease for this host. */
	if( strstr(line, "lease ") && strstr(line, "{") )
	{
	    /* Record original file pos */
	    orig_file_pos = ftell(fp);
	    file_pos = 0;
	
	    snprintf(buf, MAX_LEASE_LINE, "%s", line);

	    /* Search for buf... */	    
	    while(fgets(line, file_size, fp)!=NULL)
	    {
		if( commented(line) || strlen(line) > MAX_LEASE_LINE )
		  continue;

		/* Last lease for host will be found here */
		if( strstr(line, buf) )
		{
		    file_pos = ftell(fp);
		}
	    }
	    
	    /* Scroll to the last occurance of this lease */
	    if( file_pos != 0 )
	    {
		fseek(fp, file_pos, SEEK_SET);
		pop_lease = 0;
	    }
	    else /* None found, scroll back to the first lease */
	    {
		fseek(fp, orig_file_pos-strlen(buf), SEEK_SET);
		pop_lease = 1;
		gtk_list_store_append(GTK_LIST_STORE(widgets->lease_store), &widgets->lease_iter);
	    }

	    /* Parse and insert selected lease data */
	    while(fgets(line, file_size, fp)!=NULL)
	    {
		if( commented(line) || strlen(line) > MAX_LEASE_LINE )
		  continue;

		if( pop_lease )
		{
//		    printf("Line: %s\n", line);

		    if( strstr(line, "binding state ") && ! strstr(line, "next ") )
		      add_to_treeview(widgets, 0, line);

		    if( strstr(line, "lease ") && strstr(line, "{") )
		      add_to_treeview(widgets, 1, line);

		    if( strstr(line, "hardware ") )
		    {
		        add_to_treeview(widgets, 2, line);
		        add_to_treeview(widgets, 3, line);
		    }

		    if( strstr(line, "client-hostname ") )
		      add_to_treeview(widgets, 4, line);

		    if( strstr(line, "starts ") )
		      add_to_treeview(widgets, 5, line);

		    if( strstr(line, "ends ") )
		      add_to_treeview(widgets, 6, line);

		    if( strstr(line, "next binding state ") )
		      add_to_treeview(widgets, 7, line);

		    if( strstr(line, "cltt ") )
		      add_to_treeview(widgets, 8, line);

		    if( strstr(line, "tstp ") )
		      add_to_treeview(widgets, 9, line);

		    if( strstr(line, "tsfp ") )
		      add_to_treeview(widgets, 10, line);

		    if( strstr(line, "uuid ") )
		      add_to_treeview(widgets, 11, line);
		}
	    
		if( strstr(line, "}\n") )
		{
		    break;
		}
	    }
	    	    
	    /* Rewind to orig_file_pos and scan for the next lease */
	    fseek(fp, orig_file_pos, SEEK_SET);
	}
    }
    fclose(fp);
    free(line);
    free(buf);
}
